package me.totalfreedom.discord.util;

import discord4j.common.util.Snowflake;
import discord4j.core.object.entity.Guild;
import discord4j.core.object.entity.Member;
import discord4j.core.object.entity.Role;
import me.totalfreedom.discord.Bot;
import me.totalfreedom.totalfreedommod.TotalFreedomMod;
import me.totalfreedom.totalfreedommod.admin.Admin;
import me.totalfreedom.totalfreedommod.config.ConfigEntry;
import me.totalfreedom.totalfreedommod.player.PlayerData;
import me.totalfreedom.totalfreedommod.rank.Rank;
import me.totalfreedom.totalfreedommod.rank.Title;
import me.totalfreedom.totalfreedommod.util.FLog;
import org.bukkit.Bukkit;

public class TFM_Bridge
{
    private final Bot bot;
    private final TotalFreedomMod commons;

    public TFM_Bridge(Bot bot)
    {
        this.bot = bot;
        this.commons = (TotalFreedomMod) Bukkit.getPluginManager().getPlugin("TotalFreedomMod");

        if (commons == null) throw new IllegalStateException();
    }

    public String getDisplay(Member member)
    {
        Guild server = bot.getGuildById().block();
        // Server Owner
        if (server == null)
        {
            throw new IllegalStateException();
        }

        Snowflake ownerID = Snowflake.of(ConfigEntry.DISCORD_SERVER_OWNER_ROLE_ID.getString());
        Snowflake developerID = Snowflake.of(ConfigEntry.DISCORD_DEVELOPER_ROLE_ID.getString());
        Snowflake executiveID = Snowflake.of(ConfigEntry.DISCORD_EXECUTIVE_ROLE_ID.getString());
        Snowflake assistantExecutiveID = Snowflake.of(ConfigEntry.DISCORD_ASSISTANT_EXECUTIVE_ROLE_ID.getString());
        Snowflake seniorAdminID = Snowflake.of(ConfigEntry.DISCORD_SENIOR_ADMIN_ROLE_ID.getString());
        Snowflake adminID = Snowflake.of(ConfigEntry.DISCORD_NEW_ADMIN_ROLE_ID.getString());
        Snowflake masterBuilderID = Snowflake.of(ConfigEntry.DISCORD_MASTER_BUILDER_ROLE_ID.getString());

        Snowflake[] ids = {ownerID, developerID, executiveID, assistantExecutiveID, seniorAdminID, adminID, masterBuilderID};
        String[] titles = {Title.OWNER.getColoredTag(), Title.DEVELOPER.getColoredTag(), Title.EXECUTIVE.getColoredTag(), Title.ASSTEXEC.getColoredTag(), Rank.SENIOR_ADMIN.getColoredTag(), Rank.ADMIN.getColoredTag(), Title.MASTER_BUILDER.getColoredTag()};

        return member.getRoles().map(role ->
        {
            for (int i = 0; i < ids.length; i++)
            {
                if (role.getId().equals(ids[i]))
                {
                    return titles[i];
                }
            }
            return null;
        }).blockFirst();
    }

    public String getCode(PlayerData playerData)
    {
        for (String code : bot.getLinkCodes().keySet())
        {
            if (bot.getLinkCodes().get(code).equals(playerData))
            {
                return code;
            }
        }
        return null;
    }

    public boolean syncRoles(Admin admin, String discordID)
    {
        if (discordID == null)
        {
            return false;
        }

        Guild server = bot.getGuildById().block();

        if (server == null)
        {
            FLog.severe("The Discord server ID specified is invalid, or the bot is not on the server.");
            return false;
        }

        Member member = server.getMemberById(Snowflake.of(discordID)).block();
        if (member == null)
        {
            return false;
        }

        Role adminRole = server.getRoleById(SnowflakeEntry.adminRoleID).block();
        if (adminRole == null)
        {
            FLog.severe("The specified Admin role does not exist!");
            return false;
        }

        Role senioradminRole = server.getRoleById(SnowflakeEntry.seniorRoleID).block();
        if (senioradminRole == null)
        {
            FLog.severe("The specified Senior Admin role does not exist!");
            return false;
        }

        if (!admin.isActive())
        {
            member.getRoles()
                    .filter(role -> role.equals(adminRole) || role.equals(senioradminRole))
                    .subscribe(r -> member.removeRole(r.getId()).block());
            return true;
        }

        if (admin.getRank().equals(Rank.ADMIN))
        {
            member.getRoles()
                    .filter(role -> !role.equals(adminRole))
                    .subscribe(r -> member.addRole(r.getId()).block());
            member.getRoles()
                    .filter(role -> role.equals(senioradminRole))
                    .subscribe(r -> member.removeRole(r.getId()).block());
            return true;
        }
        else if (admin.getRank().equals(Rank.SENIOR_ADMIN))
        {
            member.getRoles()
                    .filter(role -> !role.equals(senioradminRole))
                    .subscribe(r -> member.addRole(r.getId()).block());

            member.getRoles()
                    .filter(role -> role.equals(adminRole))
                    .subscribe(r -> member.removeRole(r.getId()).block());

            return true;
        }
        return false;
    }
    public TotalFreedomMod getCommons() {
        return commons;
    }
}
